/*
 *	ashc5.c	-	MC68HC05 assembler
 *
 *	From Motorola BBS.
 *	See ../doc/ChangeLog to examine and/or add change descriptions.
 */
#include <stdlib.h>
#include <stdio.h>
#ifdef MSDOS
#	include <io.h>
#endif
#include "../inc/as.h"
#include "../inc/tablehc5.h"
#include "../inc/proto.h"
#include "as.c"
#include "dohc5.c"
#include "pseudo.c"
#include "eval.c"
#include "symtab.c"
#include "util.c"
#include "ffwd.c"
#include "output.c"

